﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectGarage.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectGarage.Repositories.Implementations;

public class RouteRepository : IRouteRepository
{
    public readonly IConnectionString _connectionString;

    private readonly ILogger<RouteRepository> _logger;

    public RouteRepository(IConnectionString connectionString, ILogger<RouteRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreateRoute(Route route)
    {
        _logger.LogInformation("Добавление объект");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(route));

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"
INSERT INTO route (RouteName, Startp, Finalp, Length) VALUES
(@RouteName, @Startp, @Finalp, @Length);";
            connection.Execute(queryInsert, route);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }
    public void UpdateRoute(Route route)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}",
        JsonConvert.SerializeObject(route));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryUpdate = @"
UPDATE route
SET
RouteName=@RouteName,
Startp=@Startp,
Finalp=@Finalp,
Length=@Length
WHERE Id=@Id";
            connection.Execute(queryUpdate, route);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }

    public void DeleteRoute(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
DELETE FROM route
WHERE Id=@id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }

    public Route ReadRouteByID(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
SELECT * FROM route
WHERE Id=@id";
            var route = connection.QueryFirst<Route>(querySelect, new { id });
            _logger.LogDebug("Найденный объект: {json}", JsonConvert.SerializeObject(route));

            return route;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }

    public IEnumerable<Route> ReadRoute(string? startPoint = null, string? finalPoint = null)
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"SELECT * FROM route";
            var routes = connection.Query<Route>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(routes));
            return routes;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }
}
