﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectGarage.Entities;
using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectGarage.Repositories.Implementations;

public class TransportationRepository : ITransportationRepository
{
    private readonly IConnectionString _connectionString;

    private readonly ILogger<TransportationRepository> _logger;

    public TransportationRepository(IConnectionString connectionString, ILogger<TransportationRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }

    public void CreateTransportation(Transportation transportation)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}",
        JsonConvert.SerializeObject(transportation));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"
INSERT INTO transportation (DriverId, RouteId, FuelId, Amount, TransportationDate)
VALUES (@DriverId, @RouteId, @FuelId, @Amount, @TransportationDate)";
            connection.Execute(queryInsert, transportation);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }

    public IEnumerable<Transportation> ReadTransportation(DateTime? dateForm = null, DateTime? dateTo = null,
        int? fuelId = null, int? driverId = null, int? routeId = null)
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            var builder = new QueryBuilder();
            if (dateForm.HasValue)
            {
                builder.AddCondition("t.TransportationDate >= @dateForm");
            }
            if (dateTo.HasValue)
            {
                builder.AddCondition("t.TransportationDate <= @dateTo");
            }
            if (fuelId.HasValue)
            {
                builder.AddCondition("t.FuelId = @fuelId");
            }
            if (driverId.HasValue)
            {
                builder.AddCondition("t.DriverId = @driverId");
            }
            if (routeId.HasValue)
            {
                builder.AddCondition("t.RouteId = @routeId");
            }
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = $@"SELECT 
    t.*,
    r.RouteName as RouteName,
    CONCAT(d.Fname, ' ', d.Lname) as DriverName,
    CONCAT(f.FuelName, ' ', f.Price) as FuelName
FROM transportation t
LEFT JOIN driver d ON d.Id = t.DriverId
LEFT JOIN fuel f ON f.Id = t.FuelId
LEFT JOIN route r ON r.Id = t.RouteId
{builder.Build()}";
            var transportations = connection.Query<Transportation>(querySelect, new {dateForm, dateTo, fuelId, driverId, routeId });
            _logger.LogDebug("Полученные объекты: {json}",
            JsonConvert.SerializeObject(transportations));
            return transportations;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }

    }
}
